% NLM_SURELET_Exhaustive
% --------------------------------------------------------------------------------
% If you use this software, please cite:
%
% D. Van De Ville, M. Kocher, 
% Non-Local Means with Dimensionality Reduction and SURE-Based Parameter Selection 
% IEEE Transactions on Image Processing, in press
% --------------------------------------------------------------------------------
%
% Run an exhaustive search over (part of) the parameter space. 
% See NLM_SURELET_Fixed.m for the meaning of the different parameters in
% the TreeParam and LeafParam options.
%
% --------------------------------------------------------------------------------

%% Init
clear
Fig = 1;
path (path, 'toolbox/');

%% Tree Parameters
SigmaNoise = 20; % [20, 50];
PCADims = input ('PCADims (NaN for variable PCA number between 6 and (2*B+1)^2) : ');
Self = input ('Include the image itself? (0=no, 1=yes) : ');

%% Leaf Parameters
Name = {'Cameraman256'}; % {'Cameraman256', 'house', 'peppers256', 'lena512'};
B = 1; % 1 : 3;
S = 5; % 4 : 13;
Lambda = .8; % .5 : .1 : 1;
MinPCANumber = 6;

%% Process
TreeParameterSet.PCAdims = PCADims;
if isnan (PCADims)
    PCADimsSize = 5; % Number of PCA trials
else
    PCADimsSize = 1;
end;
PSNR     = zeros (length(B), length(S), length(Lambda), PCADimsSize);
SUREPSNR = zeros (length(B), length(S), length(Lambda), PCADimsSize);

for iterName = 1 : length(Name)
    Name{iterName}
    
    f = double (imread (['Images/',Name{iterName},'.png']));
    [NR, NC] = size (f);
    
    for iterSigmaNoise = 1 : length (SigmaNoise)
        if SigmaNoise(iterSigmaNoise) > 0
            randn ('seed', 0);
            noise = SigmaNoise(iterSigmaNoise) * randn (size (f));
            noise = SigmaNoise(iterSigmaNoise) * noise / std(noise(:));
            fn = f + noise;
            TrueSigmaNoise = std (noise (:)); % true SigmaNoise
        end
        BestPSNR = 0;
        for iterB = 1 : length(B)
            if isnan (PCADims)
                % PCANumber = 6:(2*B(iterB)+1)^2;
                PCANumber = round (linspace (MinPCANumber, (2*B(iterB)+1)^2, PCADimsSize));
            else
                PCANumber = PCADims;
            end;
            for iterPCA = 1 : length(PCANumber)
                for iterS = 1 : length (S)
                    for iterLambda = 1 : length (Lambda)
                        [iterB, iterPCA, iterS, iterLambda]
                        if Self
                            [PSNR(iterB, iterS, iterLambda, iterPCA), SUREPSNR(iterB, iterS, iterLambda, iterPCA), Theta, FkStack, Fky, ~, ~, ~] = ...
                                surelet_nlmeans2 (f, fn, TrueSigmaNoise, [0, 0, 0, 0; B(iterB), S(iterS), Lambda(iterLambda), PCANumber(iterPCA)], TreeParameterSet);
                        else
                            [PSNR(iterB, iterS, iterLambda, iterPCA), SUREPSNR(iterB, iterS, iterLambda, iterPCA), Theta, FkStack, Fky, ~, ~, ~] = ...
                                surelet_nlmeans2 (f, fn, TrueSigmaNoise, [B(iterB), S(iterS), Lambda(iterLambda), PCANumber(iterPCA)], TreeParameterSet);
                        end;
                        % [B(iterB), S(iterS), Lambda(iterLambda), PSNR(iterB, iterS, iterLambda), SUREPSNR(iterB, iterS, iterLambda)]
                        if PSNR (iterB, iterS, iterLambda, iterPCA) > BestPSNR
                            Best.PSNR = PSNR (iterB, iterS, iterLambda, iterPCA);
                            Best.Fky = Fky;
                            Best.Theta = Theta;
                            Best.FkStack = FkStack;
                        end;
                    end;
                end;
            end;
        end;
        if Self
            save (['PerformanceVolume/PerformanceVolumeSelf_',  Name{iterName},'_',num2str(NR),'_',num2str(SigmaNoise(iterSigmaNoise)),'_',num2str(PCADims),'.mat'])
        else
            save (['PerformanceVolume/PerformanceVolumeNoSelf_',Name{iterName},'_',num2str(NR),'_',num2str(SigmaNoise(iterSigmaNoise)),'_',num2str(PCADims),'.mat'])
        end;
        Ix = find (PSNR(:) == max(PSNR(:)));
        [PSNR(Ix), SUREPSNR(Ix)]
    end;
end;