function [H,options] = perform_lowdim_embedding(M,options)

% perform_lowdim_embedding - perform a patch wise dimension extension
%
%   [H,options.P, options.Psi] = perform_lowdim_embedding(M,options);
%
%   M = perform_lowdim_embedding(H,options);
%
%   This function lift each pixel of an image (can be a color image)
%   to a vector that incorporate neighboorhood relationship.
%
%   Each pixel is replaced by the vector containing the values of the
%   neighbooring pixels and then dimension reduction is applyed to 
%   avoid manipulating very high dimensional vectors.
%
%   options.ndims gives the dimensionality for PCA.
%
%   Copyright (c) 2006 Gabriel Peyr?


dir = 1;
if size(M,3)>5
    dir = -1;
end

options.null = 0;

if isfield(options, 'k')
    k = options.k;
else
    k = 2;
end
if isfield(options, 'mask')
    mask = options.mask;
else
    mask = 'cst';
end
switch mask
    case 'cst'
        phi = ones(2*k+1);
    case 'linear'
        x = 1-abs(linspace(-1,1,2*k+3)); x = x(2:end-1);
        phi = x'*x;
end

[m,n,s] = size(M);

if dir==1
    if isfield(options, 'ndims')
        ndims = options.ndims;
    else
        ndims = 25;
    end
    % perform patch wise embedding
    s = size(M,3);
    % extract patches
    options.sampling = 'uniform';
    [H,X,Y,k] = compute_patch_library(M,k,options);
    % H = H .* repmat( phi, [1 1 s n*m] ); %DVDV - no more weighting
    % turn into collection of vectors
    H = reshape(H, [s*(2*k+1)^2 n*m]);
%     if options.moments,
%         [H,options.Harg,options.matMoments]=ComputeMoments(H,options.T);
%     end;
    
    if isinf(ndims),
        ndims = size(H,1);
        H = reshape(H, [ndims m n]);
        H = shiftdim(H,1);
        return;
    end;
    
    ndims = min(ndims,size(H,1));
end

if dir==-1,
    ndims = size(M,1);
end;

if dir==-1 
    if not(isfield(options, 'P')) || not(isfield(options, 'Psi'))
        error('You must provide options.P and options.Psi.');
    end
    P = options.P; Psi = options.Psi;
    ndims = size(P,2);
    % do un-projection
    M = reshape(M, [n^2 ndims]);
    M = shiftdim(M,1);
    M = (P*M) + repmat( Psi, [1 n^2] );
%     if options.moments,
%         M=ComputeInverseMoments(M,options.Harg,options.matMoments);
%     end;
    % sample at center
    s = options.numchannels; % number of colors
    M = reshape( M, [s (2*k+1) (2*k+1) n n] );
    M = M(:,(end+1)/2,(end+1)/2,:,:);
    H = squeeze(M);
elseif dir==1
    % compute PCA projection
    options.numchannels=size(M,3);
    if not(isfield(options, 'P')) || not(isfield(options, 'Psi')) || isempty(options.P) || isempty(options.Psi)
        Psi=mean(H,2);
        [U,S,V]=svd(H-repmat(Psi,[1 size(H,2)]),'econ');
        P=U(:,1:ndims);        
        %nbexemplars = min(n*m,5000);
        %sel = randperm(n*m);
        %sel = sel(1:nbexemplars);
        %[P,X1,v,Psi] = pca(H(:,sel),ndims);
        
    else
        P = options.P; Psi = options.Psi;
    end
    options.P=P; options.Psi=Psi;
    ndims = size(P,2);
    % perform actual PCA projection
    H = H - repmat( Psi, [1 n*m] );
    H = P'*H;
    % reshape matrix
    H = reshape(H, [ndims m n]);
    H = shiftdim(H,1);
end


