function [PSNR, SUREPSNR, Theta, FkStack, Fky, psnr_res, sure_psnr, Cond] = surelet_nlmeans2 (f, fn, TrueSigmaNoise, ParameterSet, TreeParameterSet)
% Simplified By Dimitri, just fine

%% Original size
[NR0, NC0] = size (f);
Step = NR0;
N = 1; % Dimension of the rough grid

%% MC-SURE estimate
options.do_patchwise = 0;
options.exclude_self = 0; % do not use the central pixel

K = size (ParameterSet, 1); % Number of NLM

%% Border handling!!!
max_k = max (ParameterSet (:,1));
xl=1+max_k; 
xh=size(fn,1)-max_k;
yl=1+max_k; 
yh=size(fn,2)-max_k;
NR = NR0 - 2*max_k;

%% Process
for iter = 1 : K  % Number of NLM
    % fprintf ('%03d/%03d', iter, K);
    options.k            = ParameterSet (iter, 1); % B
    options.max_dist     = ParameterSet (iter, 2); % S
    options.T            = ParameterSet (iter, 3)*TrueSigmaNoise; % SigmaEst
    if isnan (TreeParameterSet.PCAdims)
        options.ndims        = ParameterSet (iter, 4); % variable PCA number
    else
        options.ndims        = TreeParameterSet.PCAdims; % fixed PCA number
    end;        
    if options.k == 0 % exclude self
        Fk = fn;
        FkStack (:, :, iter) = fn;
        DivFkStack (:, :, iter) = ones(size(fn));
        DivFkStack(1:xl-1,:,iter)=0; DivFkStack(xh+1:end,:,iter)=0;
        DivFkStack(:,1:yl-1,iter)=0; DivFkStack(:,yh+1:end,iter)=0;
        Div(iter) = prod (size(fn(xl:xh,yl:yh)));
    else
        [Fk, Wx, Wy, options0, tmp1, tmp2, TmpDiv] = perform_nl_means (fn, options);
        clear options0
        FkStack (:, :, iter) = Fk;
        DivFkStack (:, :, iter) = TmpDiv;
        DivFkStack(1:xl-1,:,iter)=0; DivFkStack(xh+1:end,:,iter)=0;
        DivFkStack(:,1:yl-1,iter)=0; DivFkStack(:,yh+1:end,iter)=0;
        Div(iter)=sum(sum(TmpDiv(xl:xh,yl:yh)));
    end;
    
    % fprintf('\b\b\b\b\b\b\b');
    
    tmpDiff = f (xl : xh, yl : yh) - Fk (xl : xh, yl : yh);
    true_mse (iter) = norm (tmpDiff(:))^2 / (NR * NR);
    psnr_res (iter) = -10 * log10 (true_mse (iter) / 255^2);
    
    % SURE estimate
    tmpDiff = fn (xl : xh, yl : yh) - Fk (xl : xh, yl : yh);
    sure_mse (iter)     = norm (tmpDiff(:))^2 / (NR * NR) + (2 * TrueSigmaNoise^2 / (NR*NR) * Div (iter)) - TrueSigmaNoise^2; % Apply pc_factor to TrueSigmaNoise ?
    sure_mse_est (iter) = norm (tmpDiff(:))^2 / (NR * NR) + (2 * TrueSigmaNoise^2 / (NR*NR) * Div (iter)) - TrueSigmaNoise^2; % Apply pc_factor to TrueSigmaNoise ?
    sure_psnr (iter)    = -10 * log10 (sure_mse (iter) / 255^2);
end;

%% Select the best NLM instead of building a linear expansion (revision)


%% Linear system
M = N^2 * K; % length (X1Rough(:)); 

for iter1 = 1 : M
    for iter2 = 1 : M
        Phi (iter1, iter2) = sum(sum(FkStack(xl:xh,yl:yh,iter1).*FkStack(xl:xh,yl:yh,iter2)));
    end;
    WeightedDiv (:, :, iter1) = sum(sum(DivFkStack(xl:xh,yl:yh,iter1)));
    Y (iter1) = sum(sum(FkStack(xl:xh,yl:yh,iter1).*fn(xl:xh,yl:yh))) - TrueSigmaNoise^2 * WeightedDiv (:,:, iter1); % * pc_factor(iter1);
end;
Y = Y';

%% Solving the linear system
Theta = pinv (Phi) * Y;
% RCond = rcond (Phi) % Paper's revision
Cond = cond (Phi); % Paper's revision

%% Generative model F(y) = sum over j sum over k Betaj(x) Fk(y)
Fky = zeros (size (f));
LocalParameters = zeros ([size(f),K]);
WeightedDivTotal = 0.0;
for iter = 1 : M
    Fky = Fky + FkStack(:,:,iter) * Theta (iter);
    LocalParameters (:, :, iter) = LocalParameters (:, :, iter) + Theta (iter);
    WeightedDivTotal = WeightedDivTotal + Theta (iter) * sum(sum(WeightedDiv (:, :, iter))); % * pc_factor(iter);
end;

tmpImg=Fky(xl:xh,yl:yh);
tmpImg2=f(xl:xh,yl:yh);
TrueMSE = norm (tmpImg(:) - tmpImg2(:))^2 / (NR * NR); % no clipping
PSNR = -10*log10(TrueMSE/255/255);

tmpDiff=fn(xl:xh,yl:yh) - Fky(xl:xh,yl:yh);
SUREMSE = norm (tmpDiff(:))^2 / (NR * NR) + (2 * TrueSigmaNoise^2 / (NR*NR) * WeightedDivTotal) - TrueSigmaNoise^2; % How to compute this with many pc_factors ?
if SUREMSE<0,
    SUREPSNR = Inf;
else
    SUREPSNR = -10*log10(SUREMSE/255/255);
end;